
#include "RealmLocation.h"
#include "const7.h"

#include <iostream>
#include <math.h>

using namespace std;

RealmLocation::RealmLocation(long int realm, long int region, long int zone, long int scene, long int space):
	RealmHexInfo(realm),
	RegionHexInfo(region),
	ZoneHexInfo(zone),
	SceneHexInfo(scene),
	SpaceHexInfo(space)
{

}

XYLocation RealmLocation::getRealmWorldCoordinate(int zoom_precision) {

	XYLocation retxy;
	XYLocation realmxy = RealmHexInfo.getXY(isInvertedGrid(5)),
		regionxy = RegionHexInfo.getXY(isInvertedGrid(4)),
		zonexy = ZoneHexInfo.getXY(isInvertedGrid(3)),
		scenexy = SceneHexInfo.getXY(isInvertedGrid(2)),
		spacexy = SpaceHexInfo.getXY(isInvertedGrid(1));

	double precx = (
		(realmxy.getx() * getZoomScale(11) * getXScaleFactor(11)) +
		(zoom_precision <= 4 ? (regionxy.getx() * getZoomScale(10) * getXScaleFactor(10)) : 0) +
		(zoom_precision <= 3 ? (zonexy.getx() * getZoomScale(7) * getXScaleFactor(7)) : 0) +
		(zoom_precision <= 2 ? (scenexy.getx() * getZoomScale(4) * getXScaleFactor(4)) : 0)  +
		(zoom_precision <= 1 ? (spacexy.getx() * getZoomScale(1) * getXScaleFactor(1)) : 0)
					);

	double precy = (
		(realmxy.gety() * getZoomScale(11) * getYScaleFactor(11)) +
		(zoom_precision <= 4 ? (regionxy.gety() * getZoomScale(10) * getYScaleFactor(10)) : 0) +
		(zoom_precision <= 3 ? (zonexy.gety()* getZoomScale(7) * getYScaleFactor(7)) : 0) +
		(zoom_precision <= 2 ? (scenexy.gety()* getZoomScale(4) * getYScaleFactor(4)) : 0) +
		(zoom_precision <= 1 ? (spacexy.gety() * getZoomScale(1) * getYScaleFactor(1)) : 0)
					);

	//cout << "Precise world coordinate X,Y: " << precx << "," << precy << endl;

	retxy.setx(round(
		precx
	));

	retxy.sety(round(
		precy
	));

	return retxy;
}

bool RealmLocation::isInvertedGrid(int view_mode) {
	return (view_mode % 2 == 0);
}

double RealmLocation::getZoomScale(double level)
{
	double r;

	if (level == 1)
		r = 1;
	else if (level < 1)
		r = 1 / (pow(HEX_ZOOM_RATIO, (1 - level)));
	else if (level > 1)
		r = pow(HEX_ZOOM_RATIO, level - 1);

	return r;
}

double RealmLocation::getXScaleFactor(int view_mode) {
	if (!isInvertedGrid(view_mode))
		return 1.5;
	else
		return SQRT_3 / 2;
}

double RealmLocation::getYScaleFactor(int view_mode) {
	if (!isInvertedGrid(view_mode))
		return SQRT_3 / 2;
	else
		return 1.5;
}





void RealmLocation::moveSpace(int direction) {
	SpaceHexInfo = SpaceHexInfo.calcMovement(direction);
}

void RealmLocation::moveScene(int direction) {
	SceneHexInfo = SceneHexInfo.calcMovement(direction);
}

void RealmLocation::moveZone(int direction) {
	ZoneHexInfo = ZoneHexInfo.calcMovement(direction);
}

void RealmLocation::moveRegion(int direction) {
	RegionHexInfo = RegionHexInfo.calcMovement(direction);
}

void RealmLocation::moveRealm(int direction) {
	RealmHexInfo = RealmHexInfo.calcMovement(direction);
}


// finds the xy-coordinate (in hex-distance units from center) of the current location for the given zoom view mode
// (note: does not calculate distance offsets of location info on levels above or below the view mode specified)
XYLocation RealmLocation::getXY(int level) {

	switch (level) {
		case 1:
			return SpaceHexInfo.getXY(isInvertedGrid(level));
			break;
		case 2:
			return SceneHexInfo.getXY(isInvertedGrid(level));
			break;
		case 3:
			return ZoneHexInfo.getXY(isInvertedGrid(level));
			break;
		case 4:
			return RegionHexInfo.getXY(isInvertedGrid(level));
			break;
		case 5:
			return RealmHexInfo.getXY(isInvertedGrid(level));
			break;
		default:
			return XYLocation(0,0);
	}
}

// internal / private only: returns reference to relevant HexSpiralCoordinate
HexSpiralCoordinate& RealmLocation::getLevelCoordinate(int level)
{
	switch (level) {
		case 1: return SpaceHexInfo;
		case 2: return SceneHexInfo;
		case 3: return ZoneHexInfo;
		case 4: return RegionHexInfo;
		case 5: return RealmHexInfo;
	}
}

// for external use, returns HexSpiralCoordinate for given level by value
HexSpiralCoordinate RealmLocation::getHexSpiralCoordinate(int level) const
{
	switch (level) {
		case 1: return SpaceHexInfo;
		case 2: return SceneHexInfo;
		case 3: return ZoneHexInfo;
		case 4: return RegionHexInfo;
		case 5: return RealmHexInfo;
	}
}

void RealmLocation::setHexIndex(long int n, int level) {
	getLevelCoordinate(level).setHexIndex(n);
}

long int RealmLocation::getHexIndex(int level) {
	return getLevelCoordinate(level).getHexIndex();
}

void RealmLocation::normalize() {
	// todo: normalize locations that are outside the bounds of their parent, reparent to closest approximate normalized location.
}
