

#ifndef __REALMSCENEELEVATIONSUBMAP_H
	#define __REALMSCENEELEVATIONSUBMAP_H

#include "RealmSceneElevationMap.h"
#include "RealmElevationTriangle.h"
#include "RealmMapKey.h"

class RealmSceneElevationSubMap : public RealmSceneElevationMap {
	private:

	RealmElevationTriangle ElevationTriangle[6];
	RealmSceneElevationSubMap* CenterMap;
	RealmSceneElevationSubMap* NeighborMap[6];

	RealmSceneElevationMap* ParentMap;
	long int SpiralIndexInParentMap;



	int CurrentPhase;

	bool isInverted();

	void initNeighbors();
	void initTriangles(int phase = 0);

	// populates a batch of the map's elevation values
	// based on the previous phase's calculated values
	// (starting at phase 0, which is calcCornerElevations() has been called,
	// which sets up the corner values needed for phase 1 in each triangle
	//
	// * phase 0: populate corners shared between hex maps
	// * phase 0.5?: populate map + neighbor centers based on outer corners
	// * phase 1: populate triangle cores
	// * phase 2: shared triangle edge centers
	// * phase 3: populate the triangle midpoints (InnerLayer2 corners)
	// * phase 4: jump out from triangle midpoints to edge, middle dividers
	// * phase 5: InnerLayer1 key points
	// * phase 6: InnerLayer2 midpoints
	// * phase 7: fill out InnerLayer1
	// * phase 8: fill remaining shared edge points

	void populateBatch(int phase);


	public:

	// sets up elevation triangles with neighbors and their inner relationships between shared sides
	RealmSceneElevationSubMap(RealmSceneElevationMap* parent, RealmSceneElevationSubMap* centermap, RealmMapKey mapkey, RealmSceneElevationMapRepository* maprepo);

	~RealmSceneElevationSubMap() {  }

	int setSpaceFromTriangle(int hexindex, int elev) {
		return setElevation(hexindex, elev);
	}

	int findElevation(long int spiralidx);


	int getBaseVariance();

	int getCorner(int n);
	int getCornerIndex(int n);

	RealmElevationTriangle* getTriangle(int index) { return &ElevationTriangle[index]; }

	RealmSceneElevationMap* getCenterMap() { return CenterMap; }

	// calculates shared corners based on this + neighbor center values
	// void calcCornerElevations();


	void populateToPhase(int phase = 8);

	void populate();
	void populateStep();
	void populateToStep(int step) { populateToPhase(step); }

	void populateNeighborStep();
	void populateNeighbors();
};



#endif // __REALMSCENEELEVATIONSUBMAP_H

