

#ifndef __REALMSCENEKEY_H
	#define __REALMSCENEKEY_H
#include "RealmLocation.h"

struct RealmSceneKey
{
	long int realm, region, zone, scene;
	int displaylevel;

	RealmSceneKey(long int rlm, long int reg, long int zon, long int scn, long int lvl) {
		realm = rlm;
		region = reg;
		zone = zon;
		scene = scn;
		displaylevel = lvl;
	}
	RealmSceneKey(RealmLocation loc, int level) {
		realm = loc.getRealm();   // highest level of division within game world
		region = loc.getRegion(); // technically irrelevant when displaylevel > 3
		zone = loc.getZone();   // technically irrelevant when displaylevel > 2
		scene = loc.getScene(); // technically irrelevant when displaylevel > 1
		displaylevel = level;
	}
	RealmSceneKey(const RealmSceneKey& rsc) {
		realm=rsc.realm;
		region=rsc.region;
		zone=rsc.zone;
		scene=rsc.scene;
		displaylevel = rsc.displaylevel;
	}

	long int getSpiralIndex()
	{
		switch (displaylevel) {
			case 1: return scene;
			case 2: return zone;
			case 3: return region;
			case 4: return realm;
		}
		return -1;
	}

	void setSpiralIndex(long int idx) {
		switch(displaylevel) {
			case 1: scene = idx; break;
			case 2: zone = idx; break;
			case 3: region = idx; break;
			case 4: realm = idx; break;
		}
	}

	bool isInverted() const {
		return displaylevel % 2 == 0;
	}

	bool operator < (const RealmSceneKey &ob) const
	{
		return
			displaylevel > ob.displaylevel ||
			(displaylevel == ob.displaylevel && (
				realm < ob.realm ||
				(displaylevel < 4 && realm == ob.realm && region < ob.region) ||
				(displaylevel < 3 && realm == ob.realm && region == ob.region && zone < ob.zone) ||
				(displaylevel < 2 && realm == ob.realm && region == ob.region && zone == ob.zone && scene < ob.scene)
			));
	}

	bool operator == (const RealmSceneKey &ob) const
	{
		return
			(displaylevel == ob.displaylevel &&	realm == ob.realm)
			&& (
				displaylevel == 4 ||
				(displaylevel == 3 && region == ob.region ) ||
				(displaylevel == 2 && region == ob.region && zone == ob.zone) ||
				(displaylevel == 1 && region == ob.region && zone == ob.zone && scene == ob.scene)
			);
	}
};


#endif // __REALMSCENEKEY_H
