
#include <iostream>
#include "RealmSceneView.h"
#include "RealmView.h"
#include "SceneMoveAnimator.h"
#include "HexGridDisplay.h"
#include "RealmMapDisplay.h"
#include "RealmSceneElevationMap.h"
#include "RealmZoomAnimator.h"
#include "SceneCameraPositioner.h"

using namespace std;

RealmSceneView::RealmSceneView(
							int view_mode, int starting_zoom, int base_level, bool invert_base,
							RealmSceneElevationMapRepository* maprepo, RealmView* master_view
						):
	BaseLevel(base_level),
	HexGrid(
				view_mode == 1 ? RGBColor(88, 88, 88) :
				view_mode == 2 ? RGBColor(0, 255, 0) :
				view_mode == 3 ? RGBColor(255, 0, 255) :
				RGBColor(88, 0, 128),
				invert_base,
				view_mode == 4 ? 1 : 13
			)
{
	MasterView = master_view;
	ViewMode = view_mode;
	InvertedBaseGrid = invert_base;

	switch (view_mode) {
	case 1:
		HexGrid.activateOverlay1(RGBColor(255, 255, 64));
		HexGrid.activateOverlay2(RGBColor(0, 0, 255));
		HexGrid.activateOverlay3(RGBColor(0, 255, 0));
		break;
	case 2:
		HexGrid.activateOverlay1(RGBColor(255, 0, 0));
		HexGrid.activateOverlay3(RGBColor(255, 0, 255));
		break;
	case 3:
		HexGrid.activateOverlay3(RGBColor(88, 0, 128));
		break;
	case 4:
		HexGrid.activateOverlay1(RGBColor(255,255,255),false);

	}

	LastParentMark = nullptr;


	RGBColor pmark_color(200, 150, 60);

	LastParentMark = new HexGridDisplay(view_mode % 2 == 0 );
	LastParentMark->setGridSize(13);
	LastParentMark->setLineColor(pmark_color);

	SuperZoomOffset = SuperGridOffset = 3;
	SubZoomOffset = SubGridOffset = 3;

	XYLocation deadcenter(0, 0);
	SubGridPosition = deadcenter;

	using std::placeholders::_1;

	ZoomAnimator = new RealmZoomAnimator(starting_zoom);

	ZoomAnimator->setZoomFinishEvent(std::bind(&RealmSceneView::zoomFinished, this ));
	ZoomAnimator->setZoomStepEvent(std::bind(&RealmSceneView::zoomStep, this, _1));


	int grid_size = ViewMode < 4 ? 13 : 4;
	int maps_radius = 2;

	RealmSceneKey scene_center(MasterView->getCurrentLocation(), ViewMode);
	MapDisplay = new RealmMapDisplay(
								this,
								MasterView,
								maprepo,
								getBaseRadius(),

								grid_size, isInverted(), maps_radius, scene_center
							);

	CameraPositioner = new SceneCameraPositioner();
}



RealmSceneView::~RealmSceneView() {
	delete CameraPositioner;
	delete MapDisplay;
	delete LastParentMark;
	delete ZoomAnimator;
}

void RealmSceneView::addCameraAnimator(SceneMoveAnimator* animator) {
	CameraPositioner->addObjectAnimator(animator);
}

void RealmSceneView::moveStarting(HexSpiralCoordinate target) {
	MapDisplay->populateNeighborStep();
}

void RealmSceneView::moveFinished(HexSpiralCoordinate target) {
	MasterView->moveFinished(ViewMode, target);
}

void RealmSceneView::zoomStep(int progress) {
	mapPopulateTo(progress);
}

void RealmSceneView::zoomFinished() {
	if (getCurrentZoom() >= BaseLevel) {
		MapDisplay->refreshCurrentMap();
		updateBaseRadius();
	}


	MasterView->zoomFinished(ViewMode);
}

void RealmSceneView::toggleCursorCamera() {
}

void RealmSceneView::cycleSelectedObject() {
	CameraPositioner->cycleObjectAnimator();
}

void RealmSceneView::setCameraPosition(HexSpiralCoordinate newpos) {
	CameraPositioner->getTargetAnimator()->setHexPosition(newpos);
}

void RealmSceneView::toggleMapDisplay() {
	if (MapDisplay) {
		MapDisplay->toggle();
	}
}

void RealmSceneView::activateMapDisplay() {
	MapDisplay->activate();
}

void RealmSceneView::disableMapDisplay() {
	if (MapDisplay)
		MapDisplay->disable();
}

void RealmSceneView::mapPopulateStep() {
	MapDisplay->populateStep();
}

void RealmSceneView::mapPopulateTo(int level) {


	MapDisplay->populateToStep(level);
}

void RealmSceneView::mapPopulateNeighbors() {
	MapDisplay->populateNeighbors();
}

void RealmSceneView::updateMapDisplayCenter() {
	MapDisplay->updateSceneCenter();
}

void RealmSceneView::reCenterCurrentMap() {
	MapDisplay->reCenterCurrentMap();
}

void RealmSceneView::refreshCurrentMap() {
	MapDisplay->refreshCurrentMap();
}


void RealmSceneView::setSceneObjectCollection(SceneObjectCollection collection) {
	SceneObjects = collection;
	SceneObjects.updateRealmSceneView(this);
}

void RealmSceneView::updateBaseRadius() {
	MapDisplay->updateBaseRadius(getBaseRadius());
}

int RealmSceneView::getBaseRadius() {
	return MasterView->getBaseRadius() * (1 / pow(HEX_ZOOM_RATIO, BaseLevel-1) );
}

int RealmSceneView::getZoomRadius(int level, bool use_zoom_target) {
	int r = (use_zoom_target ? ZoomAnimator->getZoomScaleAtTarget(level) : getZoomScale(level) ) * MasterView->getBaseRadius();
	return r > 2 || (use_zoom_target ? ZoomAnimator->getZoomTarget() : getCurrentZoom()) > 4.5 ? (r < 1 ? 1 : r) : 2;
}

double RealmSceneView::getXScaleFactor(int level = 1) {
	if (!InvertedBaseGrid)
	{
		if (level % 2 != 0)
			return 1.5;
		else
			return SQRT_3 / 2;
	} else {
		if (level % 2 != 0)
			return SQRT_3 / 2;
		else
			return 1.5;
	}
}

double RealmSceneView::getYScaleFactor(int level = 1) {
	if (!InvertedBaseGrid)
	{
		if (level % 2 != 0)
			return SQRT_3 / 2;
		else
			return 1.5;
	} else {
		if (level % 2 != 0)
			return 1.5;
		else
			return SQRT_3 / 2;
	}
}

void RealmSceneView::startZoom(int target, int max_movement_speed) {
	ZoomAnimator->startZoom(target, max_movement_speed);

	if ((target == 1 || target == 2) && getCurrentZoom() > 1)
		refreshCurrentMap();
}

double RealmSceneView::getZoomScale(int level) {
	return ZoomAnimator->getZoomScale(level);
}
double RealmSceneView::getCurrentZoom() {
	return ZoomAnimator->getCurrentZoom();
}



// beginMovementKey to be called to start a movement action when a key is pressed and held
void RealmSceneView::beginMovementKey(int key_direction) {
	CameraPositioner->getTargetAnimator()->beginMovementKey(key_direction);
}

// stopMovementKey is called when a key is released
void RealmSceneView::stopMovementKey(int key_direction) {
	CameraPositioner->getTargetAnimator()->stopMovementKey(key_direction);
}

HexSpiralCoordinate RealmSceneView::getHexLocation() {
	return MasterView->getCurrentLocation().getHexSpiralCoordinate(ViewMode);
}

void RealmSceneView::refreshCameraPosition() {
	CameraPositioner->getTargetAnimator()->refreshPosition();
}


void RealmSceneView::updateMoveAnimation() {
	CameraPositioner->update();
}

XYLocation RealmSceneView::getScreenLocation(XYLocation hexlocation, bool use_zoom_target) {

	XYLocation screen_location;
	double zoommult;

	int zoom_radius = getZoomRadius(1, use_zoom_target);
	double xscalefactor = zoom_radius * getXScaleFactor(1);
	double yscalefactor = zoom_radius * getYScaleFactor(1);

	HexSpiralCoordinate camera_position = CameraPositioner->getHexPosition();

	HexSpiralCoordinate map_center =
				camera_position.getParentSpiralCoordinate(!isInverted())
				.getChildSpiralCoordinate(isInverted());

	HexSpiralCoordinate rebased_camera_position = camera_position - map_center;

	int base_x = rebased_camera_position.getXY(isInverted()).x * xscalefactor;
	int base_y = rebased_camera_position.getXY(isInverted()).y * yscalefactor;

	XYLocation rebased_offset = map_center.getXY(isInverted());
	int objx = (( hexlocation.x - rebased_offset.x) * xscalefactor);
	int objy = (( hexlocation.y - rebased_offset.y) * yscalefactor);

	XYLocation offset =  CameraPositioner->getCurrentOffset();

	double current_zoom = use_zoom_target ? ZoomAnimator->getZoomTarget() : getCurrentZoom();

	if (current_zoom < BaseLevel) {
		double subgrid_xscalefactor = getXScaleFactor(1 - SubGridOffset);
		double subgrid_yscalefactor = getYScaleFactor(1 - SubGridOffset);

		int subgrid_radius = getZoomRadius(1 - SubGridOffset, use_zoom_target);
		int subgridx = SubGridPosition.getx() * subgrid_xscalefactor * subgrid_radius;
		int subgridy = SubGridPosition.gety() * subgrid_yscalefactor * subgrid_radius;

		zoommult = ((BaseLevel - current_zoom) / SubZoomOffset);

		screen_location.setx(
			MasterView->getWindowCenter().getx() + objx - base_x - subgridx * zoommult - offset.x
		);
		screen_location.sety(
			MasterView->getWindowCenter().gety() + objy - base_y - subgridy * zoommult - offset.y
		);
	} else {
		zoommult = ((BaseLevel + SuperZoomOffset - current_zoom) / SuperZoomOffset);

		screen_location.setx(
			MasterView->getWindowCenter().getx() + objx - base_x * zoommult - offset.x
		);
		screen_location.sety(
			MasterView->getWindowCenter().gety() + objy - base_y * zoommult - offset.y
		);
	}

	return screen_location;
}

void RealmSceneView::update() {
	updateMoveAnimation();

	ZoomAnimator->update(MasterView->getBaseRadius());

	updateHexGrids();

	if (getCurrentZoom() >= BaseLevel)
		MapDisplay->update();
}

void RealmSceneView::updateHexGrids() {

	if (LastParentMark) {
		LastParentMark->setCenter(
			getScreenLocation(
				getHexLocation().
				getParentSpiralCoordinate(!InvertedBaseGrid).
				getChildSpiralCoordinate(InvertedBaseGrid).
				getXY(InvertedBaseGrid)
			)
		);
		LastParentMark->setHexRadius(getZoomRadius(1));
	}



	HexGrid.update(this);
}


void RealmSceneView::updateTextures(SDL_Renderer* renderer) {
	if (MapDisplay)
		MapDisplay->updateTextures(renderer);
}


void RealmSceneView::draw(SDL_Renderer* renderer) {

	if (MapDisplay)
		if (getCurrentZoom() >= BaseLevel)
			MapDisplay->draw(renderer);

	HexGrid.draw(renderer);

	if (LastParentMark && getHexLocation().getParentSpiralCoordinate(!InvertedBaseGrid).getHexIndex() != 0)
		LastParentMark->draw(renderer);


	if (getCurrentZoom() > 0 && getCurrentZoom() < 3) {
		SceneObjects.drawObjects(renderer);
	}
}
