
#ifndef __REALMSCENEVIEW_H
	#define __REALMSCENEVIEW_H

#include <SDL2/SDL.h>
#include "SceneObjectCollection.h"
#include "RealmColorMap.h"
#include "XYLocation.h"
#include "SceneHexOutline.h"
#include "HexSpiralCoordinate.h"


class RealmView;
class SceneMoveAnimator;
class HexGridDisplay;
class RealmSceneElevationMap;
class RealmMapDisplay;
class RealmSceneElevationMapRepository;
class RealmBackgroundScroller;
class RealmZoomAnimator;
class SceneCameraPositioner;

class RealmSceneView {
	private:
		// view mode in which this scene appears at this object's base zoom level (1)
		int ViewMode;
		RealmView* MasterView;

		RealmZoomAnimator* ZoomAnimator;
		int BaseLevel;

		SceneCameraPositioner* CameraPositioner;

		// Position on the sub-grid (view mode of the scene beneath this one)
		// used only when CurrentZoom < 1 (closer than this scene's view mode)
		// to adjust the subtle position toward the grid beneath
		XYLocation SubGridPosition;
		// How many zoom levels between this scene and the one specified in SubGridPosition
		int SubGridOffset;
		// optical zoom level difference between this scene and subgrid zoom
		// can be different than SubGridOffset when the BaseLevel for the level below differs
		int SubZoomOffset;

		// How many zoom levels between this and the one above
		int SuperGridOffset;
		// optical levels between this and the level above
		int SuperZoomOffset;

		SceneHexOutline HexGrid;

		RealmMapDisplay* MapDisplay;

		HexGridDisplay *LastParentMark;

		// todo move relevant parts of this into terrain object, and/or other objects into designated container groupings?
		SceneObjectCollection SceneObjects;

		bool InvertedBaseGrid;

		void refreshCurrentMap();


		void updateMoveAnimation();

		void updateHexGrids();

	public:
		RealmSceneView(
						int view_mode, int starting_zoom, int base_level, bool invert_base,
						RealmSceneElevationMapRepository* maprepo,
						RealmView* master_view
					);
		~RealmSceneView();

		RealmView* getMasterView() { return MasterView; }

		int getViewMode() { return ViewMode; }

		void addCameraAnimator(SceneMoveAnimator* animator);

		void toggleCursorCamera();
		void cycleSelectedObject();

		void setCameraPosition(HexSpiralCoordinate newpos);

		void toggleMapDisplay();
		void activateMapDisplay();
		void disableMapDisplay();

		void reCenterCurrentMap();
		void mapPopulateStep();
		void mapPopulateTo(int level);
		void mapPopulateNeighbors();
		void updateMapDisplayCenter();

		void updateBaseRadius();
		int getBaseRadius();
		int getZoomRadius(int level, bool use_zoom_target = false);

		int getBaseLevel() { return BaseLevel; }

		RealmZoomAnimator* getZoomAnimator() { return ZoomAnimator; }

		bool isInverted(int level = 1) { return (level % 2 != 0) ? InvertedBaseGrid : !InvertedBaseGrid; }
		HexSpiralCoordinate getHexLocation();

		void setSceneObjectCollection(SceneObjectCollection collection);

		// moved, now wrappers...
		void startZoom(int target, int max_movement_speed = 12);
		double getZoomScale(int level);
		double getCurrentZoom();

		double getXScaleFactor(int level);
		double getYScaleFactor(int level);

		void setSubGridPosition(XYLocation sub_grid_position) { SubGridPosition = sub_grid_position; }
		void setSubGridOffset(int sub_grid_offset) { SubGridOffset = sub_grid_offset; }
		void setSubZoomOffset(int sub_zoom_offset) { SubZoomOffset = sub_zoom_offset; }
		void setSuperGridOffset(int s) { SuperGridOffset = s; }
		void setSuperZoomOffset(int o) { SuperZoomOffset = o; }

		XYLocation getScreenLocation(XYLocation hexlocation, bool use_zoom_target = false);

		void refreshCameraPosition();


		// beginMovementKey to be called to start a movement action when a key is pressed and held
		void beginMovementKey(int key_direction);
		// stopMovement is called when a key is released
		void stopMovementKey(int key_direction);

		void moveFinished(HexSpiralCoordinate target);
		void moveStarting(HexSpiralCoordinate target);

		void zoomStep(int progress);
		void zoomFinished();

		void update();

		void updateTextures(SDL_Renderer* renderer);
		void draw(SDL_Renderer* renderer);
};

#endif // __REALMSCENEVIEW_H
