#include "SceneHexOutline.h"
#include "RealmSceneView.h"
#include <SDL2/SDL.h>
#include "HexGridDisplay.h"

SceneHexOutline::SceneHexOutline(RGBColor base_line_color, bool invert, int base_grid_size) {

	inverted = invert;
	BaseGrid = new HexGridDisplay(inverted);
	BaseGrid->setLineColor(base_line_color);
	BaseGrid->setGridSize(base_grid_size);

	OverlayGrid1 = OverlayGrid2 = OverlayGrid3 = nullptr;
}

SceneHexOutline::~SceneHexOutline() {
	delete BaseGrid;

	if (OverlayGrid1)
		delete OverlayGrid1;
	if (OverlayGrid2)
		delete OverlayGrid2;
	if (OverlayGrid3)
		delete OverlayGrid3;
}

void SceneHexOutline::activateOverlay1(RGBColor line_color, bool invert_from_base) {
	OverlayGrid1 = new HexGridDisplay(invert_from_base ? (!inverted) : inverted);
	OverlayGrid1->setLineColor(line_color);
	OverlayGrid1->setGridSize(1);
}

void SceneHexOutline::activateOverlay2(RGBColor line_color, bool invert_from_base) {
	OverlayGrid2 = new HexGridDisplay(invert_from_base ? (!inverted) : inverted);
	OverlayGrid2->setLineColor(line_color);
	OverlayGrid2->setGridSize(1);
}

void SceneHexOutline::activateOverlay3(RGBColor line_color, bool invert_from_base) {
	OverlayGrid3 = new HexGridDisplay(invert_from_base ? (!inverted) : inverted);
	OverlayGrid3->setLineColor(line_color);
	OverlayGrid3->setGridSize(0);
}

void SceneHexOutline::attachBaseColorMap(RealmColorMap* colormap) {
	BaseGrid->setFillEnabled(true);
	BaseGrid->attachColorMap(colormap);
}

void SceneHexOutline::disableBaseColorMap() {
	BaseGrid->setFillEnabled(false);
}

void SceneHexOutline::update(RealmSceneView* rsv) {

	XYLocation gridcenter = rsv->getScreenLocation(XYLocation(0, 0));

	BaseGrid->setCenter(gridcenter);
	BaseGrid->setHexRadius(rsv->getZoomRadius(1));

	if (OverlayGrid1) {
		OverlayGrid1->setCenter(gridcenter);
		OverlayGrid1->setHexRadius(rsv->getZoomRadius(2));
	}

	if (OverlayGrid2) {
		OverlayGrid2->setCenter(gridcenter);
		OverlayGrid2->setHexRadius(rsv->getZoomRadius(3));
	}

	if (OverlayGrid3) {
		OverlayGrid3->setCenter(gridcenter);
		OverlayGrid3->setHexRadius(rsv->getZoomRadius(4));
	}
}


void SceneHexOutline::draw(SDL_Renderer* renderer) {
	BaseGrid->draw(renderer);

	if (OverlayGrid1)
		OverlayGrid1->draw(renderer);
	if (OverlayGrid2)
		OverlayGrid2->draw(renderer);
	if (OverlayGrid3)
		OverlayGrid3->draw(renderer);
}
